<?php

namespace Drupal\user_provisioning\ProviderSpecific\APIHandler\UserAPIHandler;

use Drupal\Core\Config\ImmutableConfig;
use Drupal\user_provisioning\Helpers\moUserProvisioningLogger;
use Drupal\user_provisioning\ProviderSpecific\APIHandler\moAPIHandlerInterface;
use GuzzleHttp\Client;
use Psr\Http\Message\ResponseInterface;

/**
 * Class for handling API requests to configured scim application.
 */
class moUserSCIMAPIHandler implements moAPIHandlerInterface {

  /**
   * The server url.
   *
   * @var string
   */
  private string $server_url;

  /**
   * The bearer token.
   *
   * @var string
   */
  private string $bearer_token;

  /**
   * The http client.
   *
   * @var GuzzleHttp\Client
   */
  private Client $http_client;

  /**
   * The immutableconfig property.
   *
   * @var Drupal\Core\Config\ImmutableConfig
   */
  private ImmutableConfig $config;

  /**
   * The logger property.
   *
   * @var Drupal\user_provisioning\Helpers\moUserProvisioningLogger
   */
  private moUserProvisioningLogger $mo_logger;

  /**
   * Constructs a new moUserSCIMAPIHandler object.
   */
  public function __construct() {
    $this->config = \Drupal::config('user_provisioning.settings');
    $this->server_url = $this->config->get('mo_user_provisioning_scim_server_base_url');
    $this->bearer_token = $this->config->get('mo_user_provisioning_scim_server_bearer_token');
    $this->http_client = \Drupal::httpClient();
    $this->mo_logger = new moUserProvisioningLogger();
  }

  /**
   * The api get request for user entity.
   *
   * @param string $query
   *   The query part of url.
   *
   * @return \Psr\Http\Message\ResponseInterface
   *   Returns response interface
   *
   * @throws \Exception
   */
  public function get(string $query): ResponseInterface {
    $url = $this->getRequestURL($query);
    $options = ['headers' => ['Authorization' => 'Bearer ' . $this->bearer_token], 'verify' => FALSE];

    $this->mo_logger->addLog('Query url is ' . $url, __LINE__, __FUNCTION__, __FILE__);
    $this->mo_logger->addFormattedLog($options, __LINE__, __FUNCTION__, __FILE__, 'The header for resource search request is:');
    try {
      return $this->http_client->get($url, $options);
    }
    catch (\Exception $exception) {
      // 409 conflict should be handled through the catch statement.
      throw new \Exception($exception->getMessage(), $exception->getCode(), $exception);
    }
  }

  /**
   * The api post request for user entity.
   *
   * @param array $body
   *   The api request body.
   *
   * @return \Psr\Http\Message\ResponseInterface
   *   Returns response interface.
   *
   * @throws \Exception
   */
  public function post(array $body): ResponseInterface {

    $header = [
      'Accept' => 'application/json',
      'Authorization' => 'Bearer ' . $this->bearer_token,
      'Content-Type' => 'application/json',
    ];

    $url = $this->postRequestURL();

    $options = [
      'headers' => $header,
      'body' => json_encode($body),
      'verify' => FALSE,
    ];

    $this->mo_logger->addLog('Query url is ' . $url, __LINE__, __FUNCTION__, __FILE__);
    $this->mo_logger->addFormattedLog($options, __LINE__, __FUNCTION__, __FILE__, 'The header and body for resource creation request is:');

    try {
      return $this->http_client->request(
            'POST',
            $url,
            $options,
        );
    }
    catch (\Exception $exception) {
      throw new \Exception($exception->getMessage(), $exception->getCode(), $exception);
    }
  }

  /**
   * Creates and returns the URL to make get api call for user read.
   *
   * @return string
   *   Returns a get request url.
   */
  private function getRequestURL($query): string {
    return rtrim($this->server_url, '/') . '/' . $query;
  }

  /**
   * Creates and returns the URL to make POST api call for user creation.
   *
   * @return string
   *   Returns a post request url.
   */
  private function postRequestURL(): string {
    return rtrim($this->server_url, '/') . '/Users';
  }

  /**
   * {@inheritDoc}
   */
  public function put(array $body) {
    // @todo Implement put() method.
  }

  /**
   * {@inheritDoc}
   */
  public function patch(array $patch) {
    // @todo Implement patch() method.
  }

  /**
   * {@inheritDoc}
   */
  public function delete($resource_id) {
    // @todo Implement delete() method.
  }

  /**
   * {@inheritDoc}
   */
  public function deactivate(array $body) {
    // @todo Implement deactivate() method.
  }

}
